package com.lib_utils;

/**
 * 图像处理工具类
 *
 * @author Alfred_Huang
 */

import android.annotation.SuppressLint;
import android.annotation.TargetApi;
import android.content.Context;
import android.graphics.Bitmap;
import android.graphics.Bitmap.Config;
import android.graphics.BitmapFactory;
import android.graphics.Canvas;
import android.graphics.ColorMatrix;
import android.graphics.ColorMatrixColorFilter;
import android.graphics.Matrix;
import android.graphics.NinePatch;
import android.graphics.Paint;
import android.graphics.PorterDuff.Mode;
import android.graphics.PorterDuffXfermode;
import android.graphics.Rect;
import android.graphics.RectF;
import android.graphics.drawable.BitmapDrawable;
import android.graphics.drawable.Drawable;
import android.graphics.drawable.NinePatchDrawable;
import android.media.ThumbnailUtils;
import android.net.Uri;
import android.opengl.GLES10;
import android.os.Build;
import android.provider.MediaStore;
import android.renderscript.Allocation;
import android.renderscript.Element;
import android.renderscript.RenderScript;
import android.renderscript.ScriptIntrinsicBlur;
import android.util.Log;
import android.widget.LinearLayout;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.nio.ByteBuffer;

import javax.microedition.khronos.opengles.GL10;

import HPRTAndroidSDK.utils.FastBlur;

@SuppressLint("NewApi")
public class ImageUtils {

	public static int DEFAULT_IMAGE_WIDTH = 384;
	private static final int DEFAULT_MAX_BITMAP_DIMENSION = 4096;
    private static int maxHeight = 0;

	@SuppressLint("NewApi")
	public static int getBitmapSize(Bitmap bitmap) {
		if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.KITKAT) {
			// API 19
			return bitmap.getAllocationByteCount();
		}
		if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.HONEYCOMB_MR1) {
			// API 12
			return bitmap.getByteCount();
		}
		// earlier version
		return bitmap.getRowBytes() * bitmap.getHeight();
	}

	public static void setDefaultImageWidth(int defaultImageWidth) {
    		DEFAULT_IMAGE_WIDTH = defaultImageWidth;
    }

	public static int getDefaultImageWidth() {
		return DEFAULT_IMAGE_WIDTH;
	}

	/**
	 * 获得圆角图片的方法
	 *
	 * @param bitmap
	 * @param roundPx
	 * @return Bitmap CornerBitmap
	 */
	public Bitmap getRoundedCornerBitmap(Bitmap bitmap, float roundPx) {

		Bitmap output = Bitmap.createBitmap(bitmap.getWidth(), bitmap.getHeight(), Config.ARGB_8888);
		Canvas canvas = new Canvas(output);

		final int color = 0xff424242;
		final Paint paint = new Paint();
		final Rect rect = new Rect(0, 0, bitmap.getWidth(), bitmap.getHeight());
		final RectF rectF = new RectF(rect);

		paint.setAntiAlias(true);
		canvas.drawARGB(0, 0, 0, 0);
		paint.setColor(color);
		canvas.drawRoundRect(rectF, roundPx, roundPx, paint);

		paint.setXfermode(new PorterDuffXfermode(Mode.SRC_IN));
		canvas.drawBitmap(bitmap, rect, rect, paint);

		return output;
	}

	/**
	 * 将内存中的Bitmap压缩
	 *
	 * @param image
	 * @return Bitmap compressedBitmap
	 */
	public static Bitmap compressBmpFromBmp(Bitmap image) {
		ByteArrayOutputStream baos = new ByteArrayOutputStream();
		int options = 100;
		// 将Bitmap对象转换为byte数组
		image.compress(Bitmap.CompressFormat.JPEG, 100, baos);
		while (baos.toByteArray().length / 1024 > 100) {
			baos.reset();
			options -= 10;
			image.compress(Bitmap.CompressFormat.JPEG, options, baos);
		}
		ByteArrayInputStream isBm = new ByteArrayInputStream(baos.toByteArray());
		Bitmap bitmap = BitmapFactory.decodeStream(isBm, null, null);
		return bitmap;
	}

	/**
	 * 图片亮色变灰色
	 *
	 * @param drawable
	 * @return
	 */
	public static Drawable imageToGrey(Drawable drawable) {
		BitmapDrawable bd = (BitmapDrawable) drawable;
		Bitmap bitmap = bd.getBitmap();
		int width = bitmap.getWidth();
		int height = bitmap.getHeight();
		Bitmap faceIconGreyBitmap = Bitmap.createBitmap(width, height, Config.ARGB_8888);
		Canvas canvas = new Canvas(faceIconGreyBitmap);
		Paint paint = new Paint();
		ColorMatrix colorMatrix = new ColorMatrix();
		colorMatrix.setSaturation(0);
		ColorMatrixColorFilter colorMatrixFilter = new ColorMatrixColorFilter(colorMatrix);
		paint.setColorFilter(colorMatrixFilter);
		canvas.drawBitmap(bitmap, 0, 0, paint);
		BitmapDrawable bitmapDrawable = new BitmapDrawable(faceIconGreyBitmap);
		Drawable drawableGrey = bitmapDrawable;
		return drawableGrey;
	}

	/**
	 * 根据指定的图像路径和大小来获取缩略图 此方法有两点好处： 1. 使用较小的内存空间，第一次获取的bitmap实际上为null，只是为了读取宽度和高度，
	 * 第二次读取的bitmap是根据比例压缩过的图像，第三次读取的bitmap是所要的缩略图。 2.
	 * 缩略图对于原图像来讲没有拉伸，这里使用了2.2版本的新工具ThumbnailUtils，使 用这个工具生成的图像不会被拉伸。
	 *
	 * @param imagePath
	 *            图像的路径
	 * @param width
	 *            指定输出图像的宽度
	 * @param height
	 *            指定输出图像的高度
	 * @return 生成的缩略图
	 */
	public static Bitmap getImageThumbnail(String imagePath, int width, int height) {
		Bitmap bitmap = null;
		BitmapFactory.Options options = new BitmapFactory.Options();
		options.inJustDecodeBounds = true;
		// 获取这个图片的宽和高，注意此处的bitmap为null
		bitmap = BitmapFactory.decodeFile(imagePath, options);
		options.inJustDecodeBounds = false; // 设为 false
		// 计算缩放比
		int h = options.outHeight;
		int w = options.outWidth;
		int beWidth = w / width;
		int beHeight = h / height;
		int be = 1;
		if (beWidth < beHeight) {
			be = beWidth;
		} else {
			be = beHeight;
		}
		if (be <= 0) {
			be = 1;
		}
		options.inSampleSize = be;
		// 重新读入图片，读取缩放后的bitmap，注意这次要把options.inJustDecodeBounds 设为 false
		bitmap = BitmapFactory.decodeFile(imagePath, options);
		// 利用ThumbnailUtils来创建缩略图，这里要指定要缩放哪个Bitmap对象
		bitmap = ThumbnailUtils.extractThumbnail(bitmap, width, height, ThumbnailUtils.OPTIONS_RECYCLE_INPUT);
		return bitmap;
	}

	// 图片翻转
	public static Bitmap rotate90(Bitmap img) throws IOException {
		Matrix matrix = new Matrix();
		matrix.postRotate(90); /* 翻转90度 */
		int width = img.getWidth();
		int height = img.getHeight();
		Bitmap result = Bitmap.createBitmap(img, 0, 0, width, height, matrix, true);
		img.recycle();
		img = null;
		System.gc();
		return result;
	}

	/**
	 * 旋转任意角度
	 *
	 * @param bm
	 * @param degree
	 * @return
	 */
	public static Bitmap rotateBitmap(Bitmap bm, final int degree) {
		Matrix m = new Matrix();
		m.setRotate(degree, (float) bm.getWidth() / 2, (float) bm.getHeight() / 2);
		float targetX, targetY;
		if (degree == 90) {
			targetX = bm.getHeight();
			targetY = 0;
		} else {
			targetX = bm.getHeight();
			targetY = bm.getWidth();
		}

		final float[] values = new float[9];
		m.getValues(values);

		float x1 = values[Matrix.MTRANS_X];
		float y1 = values[Matrix.MTRANS_Y];

		m.postTranslate(targetX - x1, targetY - y1);

		Bitmap bmp = Bitmap.createBitmap(bm.getHeight(), bm.getWidth(), Config.ARGB_8888);
		Paint paint = new Paint();
		Canvas canvas = new Canvas(bmp);
		canvas.drawBitmap(bm, m, paint);

		return bmp;
	}
	
	/**
	 * 压缩图片至规定宽度
	 *
	 * @param bitmapIn
	 * @param width
	 * @return
	 */
	public static Bitmap compressBitmapWithWidth(Bitmap bitmapIn, int width) {
		int targetHeight = (int) ((double) (bitmapIn.getHeight() * width) / (double) bitmapIn.getWidth());
		return Bitmap.createScaledBitmap(bitmapIn, width, targetHeight, true);
	}
	
	/*public static Bitmap compressBitmapWithWidth(Bitmap bitmapIn, int width) {
		Log.i("With", "Original Height-" + bitmapIn.getHeight() + "  Width-" + bitmapIn.getWidth());
		// if (bitmapIn.getWidth() > width)
		{
			int targetWidth = width; // your arbitrary fixed limit
			int targetHeight = (int) (bitmapIn.getHeight() * targetWidth / (double) bitmapIn.getWidth());
			return Bitmap.createScaledBitmap(bitmapIn, targetWidth, targetHeight, true);
		}
		// return bitmapIn;
	}*/

	public static Bitmap compressBitmapWithDefaultWidth(Bitmap bitmapIn) {
		if (bitmapIn.getWidth() != DEFAULT_IMAGE_WIDTH) {
			int targetWidth = DEFAULT_IMAGE_WIDTH; // your arbitrary fixed limit
			int targetHeight = (int) (bitmapIn.getHeight() * targetWidth / (double) bitmapIn.getWidth());
			Bitmap result = Bitmap.createScaledBitmap(bitmapIn, targetWidth, targetHeight, true);
			/*
			 * bitmapIn.recycle(); bitmapIn = null; System.gc(); bitmapIn = result;//对原图片重新
			 */
			return result;
		}
		return bitmapIn;
	}

	public static Bitmap zoomImage(Bitmap image, double newWidth) {

		// 获取这个图片的宽和高bitmap.getWidth()
		float width = image.getWidth();
		float height = image.getHeight();
		// 如果宽度为0 保持原图
		if (newWidth == 0) {
			return image;
		}
		// your arbitrary fixed limit
		int targetWidth = (int) (width * newWidth / DEFAULT_IMAGE_WIDTH);
		int targetHeight = (int) (height * targetWidth / width);
		Bitmap mBitmap = Bitmap.createScaledBitmap(image, targetWidth, targetHeight, true);
		image.recycle();
		return mBitmap;

		/*
		 * // 创建操作图片用的matrix对象 Matrix matrix = new Matrix(); // 计算宽高缩放率 float scaleWidth
		 * = ((float) newWidth) / width; float scaleHeight = ((float) newHeight) /
		 * height; // 缩放图片动作 matrix.postScale(scaleWidth, scaleHeight); Bitmap bitmap =
		 * Bitmap.createBitmap(bgimage, 0, 0, (int) width, (int) height, matrix, true);
		 */
		// return bitmap;
	}

	/** 获取图像文件的宽高 **/
	public static int[] getImageWH(File file) {
		int[] wh = { -1, -1 };
		if (file.exists() && !file.isDirectory()) {
			try {
				BitmapFactory.Options options = new BitmapFactory.Options();
				options.inJustDecodeBounds = true;
				InputStream is = new FileInputStream(file.getAbsolutePath());
				options.inInputShareable = false;
				BitmapFactory.decodeStream(is, null, options);
				wh[0] = options.outWidth;
				wh[1] = options.outHeight;
			} catch (Exception e) {
				Log.e("TAG", "getImageWH Exception." + 3);
			}
		}
		return wh;
	}

	public static byte[] bmpToPixelsBuffer(Bitmap bmp) {
		int width = bmp.getWidth(); // 宽
		int height = bmp.getHeight(); // 高
		int count = bmp.getByteCount(); // 获取图片的RGB 565颜色数组总数

		Log.i("track", "w:" + width + "h:" + height + "hh:" + bmp.getRowBytes() + "sss:" + count);

		ByteBuffer buffer = ByteBuffer.allocate(count);
		bmp.copyPixelsToBuffer(buffer);
		byte[] data = buffer.array();
		// 获取数组
		return data;
	}

	public static byte[] bitmapToByteArray(Bitmap bmp) {
		if (bmp != null) {
			ByteArrayOutputStream baos = new ByteArrayOutputStream();
			bmp.compress(Bitmap.CompressFormat.PNG, 100, baos);
			try {
				baos.flush();
				baos.close();
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			return baos.toByteArray();
		}
		return null;
	}

	public static Bitmap setDefaultImageWidth(int width, int height, byte[] byteArray) {
		Bitmap stitchBmp = Bitmap.createBitmap(width, height, Config.ARGB_8888);

		stitchBmp.copyPixelsFromBuffer(ByteBuffer.wrap(byteArray));

		return stitchBmp;
	}

	public static int[] bitmapToIntArray(Bitmap bmp) {
		int[] pixels = new int[bmp.getWidth() * bmp.getHeight()];
		bmp.getPixels(pixels, 0, bmp.getWidth(), 0, 0, bmp.getWidth(), bmp.getHeight());
		// for(int i = 0; i < pixels.length; i++){
		// int clr = pixels[i];
		// int red = (clr & 0x00ff0000) >> 16; //取高两位
		// int green = (clr & 0x0000ff00) >> 8; //取中两位
		// int blue = clr & 0x000000ff; //取低两位
		//
		// }
		return pixels;

	}

	public static Bitmap byteArrayToBitmap(byte[] bytes) {
		if (bytes.length != 0) {

			return BitmapFactory.decodeByteArray(bytes, 0, bytes.length);
		} else {
			return null;
		}
	}

	public static Bitmap compressImage(Bitmap image, int quality) {
		BitmapFactory.Options options = new BitmapFactory.Options();
		// 节约内存
		/* 降低图片从ARGB888到RGB565 */
		options.inPreferredConfig = Config.RGB_565;
		/* 不进行图片抖动处理 */
		options.inDither = false;
		/* 设置让解码器以最佳方式解码 */
		options.inPreferredConfig = null;
		/* 下面两个字段需要组合使用 */
		options.inPurgeable = true;
		options.inInputShareable = true;

		ByteArrayOutputStream baos = new ByteArrayOutputStream();
		// 质量压缩方法，这里100表示不压缩，把压缩后的数据存放到baos中
		image.compress(Bitmap.CompressFormat.JPEG, quality, baos);
		// 把压缩后的数据baos存放到ByteArrayInputStream中
		ByteArrayInputStream isBmp = new ByteArrayInputStream(baos.toByteArray());
		// 把ByteArrayInputStream数据生成图片
		Bitmap bitmap = BitmapFactory.decodeStream(isBmp, null, options);
		return bitmap;
	}

	/**
	 * 压缩图片
	 *
	 * @param image
	 * @return
	 */
	public static Bitmap compressImage(Bitmap image) {
		BitmapFactory.Options options = new BitmapFactory.Options();
		// 节约内存
		/* 降低图片从ARGB888到RGB565 */
		options.inPreferredConfig = Config.RGB_565;
		/* 不进行图片抖动处理 */
		options.inDither = false;
		/* 设置让解码器以最佳方式解码 */
		options.inPreferredConfig = null;
		/* 下面两个字段需要组合使用 */
		options.inPurgeable = true;
		options.inInputShareable = true;

		ByteArrayOutputStream baos = new ByteArrayOutputStream();
		// 质量压缩方法，这里100表示不压缩，把压缩后的数据存放到baos中
		image.compress(Bitmap.CompressFormat.JPEG, 100, baos);

		int quality = 90;
		// 循环判断如果压缩后图片是否大于150kb,大于继续压缩
		while (baos.toByteArray().length / 1024 > 150) {
			// 重置baos
			baos.reset();
			// 这里压缩options%，把压缩后的数据存放到baos中
			image.compress(Bitmap.CompressFormat.JPEG, quality, baos);
			// 每次都减少10
			quality -= 10;
			// Log.e("compress", "quality = " + quality + ", byte size = " +
			// baos.toByteArray().length / 1024 + "k");
			if (quality < 10) {
				break;
			}
		}
		// 把压缩后的数据baos存放到ByteArrayInputStream中
		ByteArrayInputStream isBm = new ByteArrayInputStream(baos.toByteArray());
		// 把ByteArrayInputStream数据生成图片
		Bitmap bitmap = BitmapFactory.decodeStream(isBm, null, options);
		// Bitmap bitmap = BitmapFactory.decodeStream(isBm, null, null);
		return bitmap;
	}

	public static Bitmap compressRatioAndQuality(Bitmap image) {

		ByteArrayOutputStream baos = new ByteArrayOutputStream();
		image.compress(Bitmap.CompressFormat.JPEG, 100, baos);
		if (baos.toByteArray().length / 1024 > 1024) {// 判断如果图片大于1M,进行压缩避免在生成图片（BitmapFactory.decodeStream）时溢出
			baos.reset();// 重置baos即清空baos
			image.compress(Bitmap.CompressFormat.JPEG, 50, baos);// 这里压缩50%，把压缩后的数据存放到baos中
		}
		ByteArrayInputStream isBm = new ByteArrayInputStream(baos.toByteArray());
		BitmapFactory.Options newOpts = new BitmapFactory.Options();
		// 开始读入图片，此时把options.inJustDecodeBounds 设回true了
		newOpts.inJustDecodeBounds = true;
		Bitmap bitmap = BitmapFactory.decodeStream(isBm, null, newOpts);
		newOpts.inJustDecodeBounds = false;
		int w = newOpts.outWidth;
		int h = newOpts.outHeight;
		// 现在主流手机比较多是800*480分辨率，所以高和宽我们设置为
		float hh = 800f;// 这里设置高度为800f
		float ww = 480f;// 这里设置宽度为480f
		// 缩放比。由于是固定比例缩放，只用高或者宽其中一个数据进行计算即可
		int be = 1;// be=1表示不缩放
		if (w > h && w > ww) {// 如果宽度大的话根据宽度固定大小缩放
			be = (int) (newOpts.outWidth / ww);
		} else if (w < h && h > hh) {// 如果高度高的话根据宽度固定大小缩放
			be = (int) (newOpts.outHeight / hh);
		}
		if (be <= 0)
			be = 1;
		newOpts.inSampleSize = be;// 设置缩放比例
		// 重新读入图片，注意此时已经把options.inJustDecodeBounds 设回false了
		isBm = new ByteArrayInputStream(baos.toByteArray());
		bitmap = BitmapFactory.decodeStream(isBm, null, newOpts);
		return compressRatioAndQuality(bitmap);// 压缩好比例大小后再进行质量压缩
	}

	public static Bitmap setImageMatchParent(LinearLayout view, Bitmap bitmap) {
		int bmpWidth = bitmap.getWidth();
		int bmpHeight = bitmap.getHeight();
		int width = view.getWidth();
		int height = view.getHeight();
		float ratio = (float) width / (float) bmpWidth;
		Matrix matrix = new Matrix();
		if (width > bmpWidth) {
			matrix.postScale(ratio, ratio);
		} else {
			matrix.postScale(1 / ratio, 1 / ratio);
		}
		// if (bitmap != null && bitmap.isRecycled()) {
		// bitmap.recycle();
		// }

		// 根据缩放比例获取新的位图
		Bitmap dstbmp = Bitmap.createBitmap(bitmap, 0, 0, bmpWidth, bmpHeight, matrix, true);
		return dstbmp;
	}

	@TargetApi(Build.VERSION_CODES.JELLY_BEAN_MR1)
	public static Bitmap gaussianBlur(Context context, Bitmap bitmap, int quality, int width, float radius) {
		Bitmap compressBitmap = compressImage(bitmap, quality);
		bitmap = compressBitmapWithWidth(compressBitmap, width);
		Bitmap bmp = bitmap.copy(bitmap.getConfig(), true);
		final RenderScript rs = RenderScript.create(context);
		final Allocation input = Allocation.createFromBitmap(rs, bmp, Allocation.MipmapControl.MIPMAP_NONE,
				Allocation.USAGE_SCRIPT);
		final Allocation output = Allocation.createTyped(rs, input.getType());
		final ScriptIntrinsicBlur script = ScriptIntrinsicBlur.create(rs, Element.U8_4(rs));
		script.setRadius(radius); /* e.g. 3.f */
		script.setInput(input);
		script.forEach(output);
		output.copyTo(bitmap);
		return bitmap;
	}

	public static Bitmap gaussianBlur(Bitmap bitmap) {
		float radius = 10;
		float scaleFactor = 16;

		Bitmap overlay = Bitmap.createBitmap((int) (bitmap.getWidth() / scaleFactor),
				(int) (bitmap.getHeight() / scaleFactor), Config.ARGB_8888);
		Canvas canvas = new Canvas(overlay);
		canvas.scale(1 / scaleFactor, 1 / scaleFactor);
		Paint paint = new Paint();
		paint.setFlags(Paint.FILTER_BITMAP_FLAG);
		canvas.drawBitmap(bitmap, 0, 0, paint);
		overlay = FastBlur.doBlur(overlay, (int) radius, true);
		return overlay;
	}

	public static Bitmap gaussianBlur(Context context, Bitmap bitmap) {
		return gaussianBlur(context, bitmap, 0, 100, 25);
	}

	/**
	 * 说明：即使这样，你会发现点9的右线和低线的文字区域还是不管用，不过这已经不影响我们使用啦。
	 *
	 * @param bitmap
	 * @param context
	 * @return
	 */
	public static Drawable getNinePatchDrawable(Bitmap bitmap, Context context) {

		byte[] chunk = bitmap.getNinePatchChunk();
		NinePatchDrawable ninePatchDrawable = null;
		if (NinePatch.isNinePatchChunk(chunk)) {
			ninePatchDrawable = new NinePatchDrawable(context.getResources(), bitmap, chunk, new Rect(), null);
		}
		return ninePatchDrawable;
	}
	
    static {
        int[] maxTextureSize = new int[1];
        GLES10.glGetIntegerv(GL10.GL_MAX_TEXTURE_SIZE, maxTextureSize, 0);
        maxHeight = Math.max(maxTextureSize[0], DEFAULT_MAX_BITMAP_DIMENSION);
    }

    public static int getMaxHeight(){
        return maxHeight;
    }

    public static Bitmap getBitmapByUri(Context context, Uri uri) {
        Bitmap bitmap = null;
        try {
            // 根据Uri获取Bitmap图片的静态方法
            Bitmap image = MediaStore.Images.Media.getBitmap(context.getContentResolver(), uri);
            if (image != null) {
                bitmap = image;
            }
        } catch (IOException e) {
            e.printStackTrace();
        }
        return bitmap;
    }

}
